package net.sf.albertedm.mojoplugin;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.Map.Entry;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.handler.ArtifactHandler;
import org.apache.maven.plugin.logging.Log;
import org.apache.maven.project.MavenProject;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;
import org.xml.sax.SAXException;

public class GenerateJnlp {

	// TAGS de Elementos
	private static final String ELEMENT_JNLP = "jnlp";
	private static final String ELEMENT_INFORMATION = "information";
	private static final String ELEMENT_TITLE = "title";
	private static final String ELEMENT_VENDOR = "vendor";
	private static final String ELEMENT_HOMEPAGE = "homepage";
	private static final String ELEMENT_DESCRIPTION = "description";
	private static final String ELEMENT_ICON = "icon";
	private static final String ELEMENT_SHORTCUT = "shortcut";
	private static final String ELEMENT_DESKTOP = "desktop";
	private static final String ELEMENT_MENU = "menu";
	private static final String ELEMENT_UPDATE = "update";
	private static final String ELEMENT_SECURITY = "security";
	private static final String ELEMENT_ALL_PERMISSIONS = "all-permissions";
	private static final String ELEMENT_APLICATION_DESC = "application-desc";
	private static final String ELEMENT_RESOURCES = "resources";
	private static final String ELEMENT_PROPERTY = "property";
	private static final String ELEMENT_J2SE = "j2se";
	private static final String ELEMENT_JAR = "jar";
	private static final String ELEMENT_NATIVELIB = "nativelib";

	// TAGS de Atributos
	private static final String ATTRIBUTE_SPEC = "spec";
	private static final String ATTRIBUTE_CODEBASE = "codebase";
	private static final String ATTRIBUTE_HREF = "href";
	private static final String ATTRIBUTE_VERSION = "version";
	private static final String ATTRIBUTE_KIND = "kind";
	private static final String ATTRIBUTE_ONLINE = "online";
	private static final String ATTRIBUTE_SUBMENU = "submenu";
	private static final String ATTRIBUTE_CHECK = "check";
	private static final String ATTRIBUTE_MAIN_CLASS = "main-class";
	private static final String ATTRIBUTE_NAME = "name";
	private static final String ATTRIBUTE_VALUE = "value";
	private static final String ATTRIBUTE_JAVA_VM_ARGS = "java-vm-args";

	// Valores especiales (entiendo que fijos)
	private static final String VALUE_SHORT = "short";
	private static final String VALUE_UPDATE_CHECK = "always";

	private final MavenProject project;
	private final Map<String, String> codebasesByEnvironment;
	private final String mainClass;

	private String title = null;
	private String vendor = null;
	private String homepage = null;
	private String description = null;
	private String shortDescription = null;
	private String iconRef = null;
	private boolean shortcutOnline = true;
	private String submenu = null;
	private String jnlpName = "run.jnlp";
	private String jnlpFolder = ".";
    private String confFolder = "conf";
    private String libFolder = "lib";
    private String logProperties = null;
    private String javaVMArgs = null;
    private String j2seVersion = null;

	public GenerateJnlp(MavenProject project,
			Map<String, String> codebasesByEnvironment, String mainClass) {
		super();
		// TODO Validar argumentos
		this.project = project;
		this.codebasesByEnvironment = codebasesByEnvironment;
		this.mainClass = mainClass;
	}

	/**
	 * @param args
	 * arg[0] = environment
	 * arg[1] = version
	 * @throws IOException
	 */
	public void generate(Log log, String environment) throws SAXException, IOException {

        // Lectura de Parametros
	    String name = project.getName();
	    String version = project.getVersion();

		String outputFile = project.getBasedir().getCanonicalPath() + File.separator + jnlpFolder + File.separator + jnlpName;
		String spec = "1.0+";
		String codebase = codebasesByEnvironment.get(environment);

		Map<String,String> properties = new HashMap<String, String>();
		properties.put("propertiesConfDirectory", codebase + confFolder);
		properties.put("log4j.configuration", codebase + logProperties);
		properties.put("appVersion", version);

		String mainJar = name + "-" + version + "." + project.getPackaging();

        // Rellenamos las dependencias
		Set<String> nativeJars = new TreeSet<String>();
		Set<String> libJars = new TreeSet<String>();
		fillJars(project, mainJar, nativeJars, libJars);

		// Generacion del XML
        Document document = DocumentHelper.createDocument();

        // Elemento Raiz
        Element root = document.addElement(ELEMENT_JNLP)
        	.addAttribute(ATTRIBUTE_SPEC, spec)
        	.addAttribute(ATTRIBUTE_CODEBASE, codebase)
        	.addAttribute(ATTRIBUTE_HREF, jnlpName)
        	.addAttribute(ATTRIBUTE_VERSION, version);

        // Elemento Information
        Element information = root.addElement(ELEMENT_INFORMATION);
        fillInformation(information);

        // Otros elementos
        root.addElement(ELEMENT_UPDATE).addAttribute(ATTRIBUTE_CHECK, VALUE_UPDATE_CHECK);
		// TODO Merecera la pena hacer algo con los permisos?
        root.addElement(ELEMENT_SECURITY).addElement(ELEMENT_ALL_PERMISSIONS);
        root.addElement(ELEMENT_APLICATION_DESC).addAttribute(ATTRIBUTE_MAIN_CLASS, mainClass);

        // Elemento Resources
        Element resources = root.addElement(ELEMENT_RESOURCES);
        fillResources(resources, properties, mainJar, nativeJars, libJars);

        write(document, outputFile);
	}

	private void fillInformation(Element information) {

		if(title != null) {
        	information.addElement(ELEMENT_TITLE).addText(title);
        }
        if(vendor != null) {
            information.addElement(ELEMENT_VENDOR).addText(vendor);
        }
        if(homepage != null) {
            information.addElement(ELEMENT_HOMEPAGE).addAttribute(ATTRIBUTE_HREF, homepage);
        }
        if(description != null) {
            information.addElement(ELEMENT_DESCRIPTION).addText(description);
            if(shortDescription != null) {
                information.addElement(ELEMENT_DESCRIPTION).addAttribute(ATTRIBUTE_KIND, VALUE_SHORT).addText(shortDescription);
            }
        }
        if(iconRef != null) {
            information.addElement(ELEMENT_ICON).addAttribute(ATTRIBUTE_HREF, iconRef);
        }
        Element shortcut = information.addElement(ELEMENT_SHORTCUT).addAttribute(ATTRIBUTE_ONLINE, Boolean.toString(shortcutOnline));
        if(shortcutOnline) {
	        shortcut.addElement(ELEMENT_DESKTOP);
	        if(submenu != null) {
		        shortcut.addElement(ELEMENT_MENU).addAttribute(ATTRIBUTE_SUBMENU, submenu);
	        }
        }
		
	}

	private void fillResources(Element resources, Map<String, String> properties, String mainJar,
			Collection<String> nativeJars, Collection<String> libJars) {
		
        for(Entry<String, String> property : properties.entrySet()) {
        	resources.addElement(ELEMENT_PROPERTY)
        		.addAttribute(ATTRIBUTE_NAME, property.getKey())
        		.addAttribute(ATTRIBUTE_VALUE, property.getValue());
        }
        resources.addElement(ELEMENT_J2SE).addAttribute(ATTRIBUTE_VERSION, j2seVersion).addAttribute(ATTRIBUTE_JAVA_VM_ARGS, javaVMArgs);
        // Main JAR
        resources.addElement(ELEMENT_JAR).addAttribute(ATTRIBUTE_HREF, libFolder + "/" + mainJar);
        // Native JARS
        for(String nativeJar : nativeJars) {
        	resources.addElement(ELEMENT_NATIVELIB)
        		.addAttribute(ATTRIBUTE_HREF, libFolder + "/" + nativeJar);
        }
        // Other JARS
        for(String libJar : libJars) {
        	resources.addElement(ELEMENT_JAR)
        		.addAttribute(ATTRIBUTE_HREF, libFolder + "/" + libJar);
        }
	}

	private static void fillJars(MavenProject mavenProject, String mainJar,
			Set<String> nativeJars, Set<String> libJars) throws IOException {
		
		for(Object artifactObject : mavenProject.getArtifacts()) {
		
			Artifact artifact = (Artifact) artifactObject;
		    ArtifactHandler handler = artifact.getArtifactHandler();
	        String jarName = artifact.getArtifactId() + "-" + artifact.getVersion() + "." + handler.getExtension();
	        boolean isNative = false;
	        JarFile jarFile = new JarFile(artifact.getFile());
            Enumeration<JarEntry> entries = jarFile.entries();
            
            while (!isNative && entries.hasMoreElements()) {
                JarEntry entry = entries.nextElement();
                String entryName = entry.getName();
                if(entryName.endsWith(".dll") || entryName.endsWith(".so")) {
                    isNative = true;
                }
            }
            
            jarFile.close();
	        if(isNative) {
	            nativeJars.add(jarName);
	        }
	        else {
	            if(!mainJar.equals(jarName)) {
	                libJars.add(jarName);
	            }
	        }
		}
	}

	public static void write(Document document, String filePath) throws IOException {

	    File outputFile = new File(filePath);
	    if(outputFile.exists()) {
	        outputFile.renameTo(new File(outputFile.getCanonicalPath() + ".old"));
	    }

        OutputFormat format = OutputFormat.createPrettyPrint();
        format.setNewLineAfterNTags(1);
        format.setNewLineAfterDeclaration(false);
        format.setIndent("    ");

        XMLWriter writer = new XMLWriter(
            new FileWriter(filePath), format
        );
        writer.write(document);
        writer.close();

        // Pretty print the document to System.out
        writer = new XMLWriter(System.out, format);
        writer.write(document);

    }

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getVendor() {
		return vendor;
	}

	public void setVendor(String vendor) {
		this.vendor = vendor;
	}

	public String getHomepage() {
		return homepage;
	}

	public void setHomepage(String homepage) {
		this.homepage = homepage;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getShortDescription() {
		return shortDescription;
	}

	public void setShortDescription(String shortDescription) {
		this.shortDescription = shortDescription;
	}

	public String getIconRef() {
		return iconRef;
	}

	public void setIconRef(String iconRef) {
		this.iconRef = iconRef;
	}

	public boolean isShortcutOnline() {
		return shortcutOnline;
	}

	public void setShortcutOnline(boolean shortcutOnline) {
		this.shortcutOnline = shortcutOnline;
	}

	public String getSubmenu() {
		return submenu;
	}

	public void setSubmenu(String submenu) {
		this.submenu = submenu;
	}

    public MavenProject getProject() {
        return project;
    }

	public String getMainClass() {
		return mainClass;
	}

	public Map<String, String> getCodebasesByEnvironment() {
		return codebasesByEnvironment;
	}

    public String getJnlpName() {
        return jnlpName;
    }

    public void setJnlpName(String jnlpName) {
        this.jnlpName = jnlpName;
    }

    public String getJnlpFolder() {
        return jnlpFolder;
    }

    public void setJnlpFolder(String jnlpFolder) {
        this.jnlpFolder = jnlpFolder;
    }

    public String getConfFolder() {
        return confFolder;
    }

    public void setConfFolder(String confFolder) {
        this.confFolder = confFolder;
    }

    public String getLibFolder() {
        return libFolder;
    }

    public void setLibFolder(String libFolder) {
        this.libFolder = libFolder;
    }

    public String getLogProperties() {
        return logProperties;
    }

    public void setLogProperties(String logProperties) {
        this.logProperties = logProperties;
    }

    public String getJ2seVersion() {
        return j2seVersion;
    }

    public void setJ2seVersion(String j2seVersion) {
        this.j2seVersion = j2seVersion;
    }

    public String getJavaVMArgs() {
        return javaVMArgs;
    }

    public void setJavaVMArgs(String javaVMArgs) {
        this.javaVMArgs = javaVMArgs;
    }
}
