package net.sf.albertedm.mojoplugin;

import java.io.IOException;
import java.util.Map;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.logging.Log;
import org.apache.maven.project.MavenProject;
import org.xml.sax.SAXException;

/**
 * Goal which generates a jnlp file.
 *
 * @goal jnlp
 *
 * @requiresDependencyResolution runtime
 * @requiresDependencyCollection compile
 *
 * @phase validate
 */
public class GenerateJnlpMojo extends AbstractMojo {

    /**
     * Location of the file.
     * @parameter expression="${project}"
     * @required
     */
    private MavenProject project;

    /**
     * Environment
     * @parameter expression="${environment}"
     * @required
     */
    private String environment;

    /**
     * Module where run the plugin
     *
     * @parameter
     * @required
    */
    private String targetModule;

    /**
     * Main class to launch the JNLP application
     *
     * @parameter
     * @required
    */
    private String mainClassName;

    /**
     * Codebases to be used which each environment
     *
     * @parameter
     * @required
    */
    private Map<String, String> codebaseByEnvironment;

    /**
     * Title that will appear when running the application
     *
     * @parameter
    */
    private String title;

    /**
     * Vendor of the application
     *
     * @parameter
    */
    private String vendor;

    /**
     * Business Home Page
     *
     * @parameter
    */
    private String homepage;

    /**
     * Description of the application
     *
     * @parameter
    */
    private String description;

    /**
     * Brief description of the application
     *
     * @parameter
    */
    private String shortDescription;

    /**
     * Resource name of the icon of the application
     *
     * @parameter
    */
    private String iconRef;

    /**
     * Whether to establish a desktop shortcut of the application
     *
     * @parameter default-value="true"
    */
    private boolean shortcutOnline;

    /**
     * Submenu where include the application
     *
     * @parameter
    */
    private String submenu;

    /**
     * Name for the output jnlp file
     *
     * @parameter
    */
    private String jnlpName;

    /**
     * Folder where save the jnlp file
     *
     * @parameter
    */
    private String jnlpFolder;

    /**
     * Folder with configuration files
     *
     * @parameter
    */
    private String confFolder;

    /**
     * Folder where classpath libraries will be accessible
     *
     * @parameter
    */
    private String libFolder;

    /**
     * Log configuration file
     *
     * @parameter
    */
    private String logProperties;

    /**
     * J2SE version.
     *
     * @parameter
     * @required
    */
    private String j2seVersion;

    /**
     * Parameters for Java Virtual Machine
     *
     * @parameter
    */
    private String javaVMArgs;

    public void execute() throws MojoExecutionException {
        
    	Log log = getLog();

    	log.info("Starting goal jnlp...");

    	if(targetModule.equals(project.getName())) {

    	    GenerateJnlp generateJnlp = new GenerateJnlp(
    	            project,
    	            codebaseByEnvironment,
    	            mainClassName);
    	    
   	        generateJnlp.setTitle(title);
   	        generateJnlp.setVendor(vendor);
   	        generateJnlp.setHomepage(homepage);
   	        generateJnlp.setDescription(description);
   	        generateJnlp.setShortDescription(shortDescription);
   	        generateJnlp.setIconRef(iconRef);
    	    generateJnlp.setShortcutOnline(shortcutOnline);
            generateJnlp.setSubmenu(submenu);
            generateJnlp.setJnlpName(jnlpName);
            generateJnlp.setJnlpFolder(jnlpFolder);
            generateJnlp.setConfFolder(confFolder);
            generateJnlp.setLibFolder(libFolder);
            generateJnlp.setLogProperties(logProperties);
            generateJnlp.setJ2seVersion(j2seVersion);
            generateJnlp.setJavaVMArgs(javaVMArgs);

    	    try {
   	            generateJnlp.generate(log, environment);
    	    } catch (SAXException e) {
    	        log.error(e.getMessage(), e);
    	    } catch (IOException e) {
    	        log.error(e.getMessage(), e);
    	    }
    	}
    	
    	log.info("Finished goal jnlp");
    }
}
